type t 

(** Declarations form type *)
val input : string -> Type.t -> Speculog.declaration * t 
val output : string -> Type.t -> Speculog.declaration * t 
val reg : string -> Type.t -> Speculog.declaration * t 
val wire : string -> Type.t -> Speculog.declaration * t 
val var : string -> Type.t -> t

(** Constructors *)
val unit : t
val bool : bool -> t
val int : int -> t
val array : t array -> t
val record : (string * t) list -> t
val constr : Type.t -> string -> t -> t


(** Access *)

(** Should only be applied to simple types (bool and int) *)
val to_expr : t -> unit Integer.t

(** Should only be applied to arrays.
    The first argument is a table and
    the second argument is the index of the element we want. *)
val get : t -> t -> t

(** Should only be applied to tuples *)
val field : t -> string -> t

(** Should only be applied to union types *)
val match_with : Type.t -> t -> (string * (t -> t)) list -> t
val match_case : Type.t -> t -> string -> Type.t * t

(** Operations *)
val neg : t -> t
val implies : t -> t -> t
val equiv : t -> t -> t
val conj : t -> t -> t
val disj : t -> t -> t
val xor : t -> t -> t
val equals : t -> t -> t
val less_eq : t -> t -> t
val less : t -> t -> t
val greater_eq : t -> t -> t
val greater : t -> t -> t
val left_shift : t -> int -> t
val right_shift : t -> int -> t 
val add : t -> t -> t
val minus : t -> t -> t
val mult : t -> t -> t
val div : t -> t -> t
val modulo : t -> t -> t
val next : t -> t

val ite : t -> t -> t -> t

val apply : (unit Integer.t -> unit Integer.t -> unit Integer.t) -> t -> t -> t


(** Conversions *)
val to_string : t -> string 
val to_int : Type.t -> t -> t
val of_int : Type.t -> t -> t
  

(** Add an initial configuration to the specifications. *)
val init : (t * t) list -> (t * t) list -> (t * t) list
val functional_synthesis : (t * t) list -> Aiger.t
(** Should be applied to Boolean expressions *)
val for_each : (int * int) list -> (int -> t) -> t
val for_some : (int * int) list -> (int -> t) -> t
