(** Parse an aiger file and return two lists, one containing the controllables inputs and the other the uncontrollable inputs. *)
val read_from_file : string -> (Aiger.t * Aiger.lit list * Aiger.lit list)
val of_aiger : Aiger.t -> (Aiger.t * Aiger.lit list * Aiger.lit list)

val controllable_name : string -> bool
val controllables : Aiger.t -> (Aiger.lit list * Aiger.lit list)

(** type for attractor regions *)
type t 

(** configurations of the latches (corresponds to player 1 states) *)
val latch_configuration : t -> Cudd.bdd
(** latches and inputs that are in the region (corresponds to player 2 states) *)
val latch_input_configuration : t -> Cudd.bdd

val negation : t -> t
val diff : t -> t -> t
val intersection : t -> t -> t
val intersection_bdd : t -> Cudd.bdd -> t
val union : t -> t -> t 
val equal : t -> t -> bool
(** everything *)
val tt : unit -> t
(** nothing *)
val ff : unit -> t 

(** the first BDD gives restriction on the action of the players playing first and the second BDD on the actions of the player playing second *)
val of_bdds : Cudd.bdd -> Cudd.bdd -> t

(** [attractor aiger controllables uncontrollables goal] 
    Compute the set of states from which the controller can force going to the goal states. By default weak is set to false which means that controller is player 2 
*)
val attractor : AigerBdd.t -> Aiger.lit list -> Aiger.lit list -> ?weak:bool -> Cudd.bdd -> t

(** [trap aiger controllables uncontrollables failure] 
    Compute the set of states from which the controller cannot avoid going to the failure states. If the weak flag is set to true, this means that the controller cannot see the uncontrollable actions before taking its decision (ie controller = player 2).
*)
val trap : AigerBdd.t -> Aiger.lit list -> Aiger.lit list -> ?weak:bool -> Cudd.bdd -> t

(** Compute the same operator as the previous function, except that the actions are restricted by the last argument *)
val attractor_with_restriction : AigerBdd.t -> Aiger.lit list -> Aiger.lit list -> ?weak:bool -> t -> t -> t

(** Compute the same operator as the previous function, except that the first BDD gives restriction on the actions of the controller and the environment *)
val trap_with_restriction : AigerBdd.t -> Aiger.lit list -> Aiger.lit list -> ?weak:bool -> t -> t -> t

val strategy : AigerBdd.t -> t -> Cudd.bdd

val strategy_to_aiger : Aiger.t -> Cudd.bdd -> Aiger.lit list -> Aiger.lit list -> Aiger.t

(** Given an aiger file, check that there is a safe strategy to avoid having one output to true. *)
val test : Aiger.t -> unit
