(* Game specified by non-deterministic Büchi conditions *)
(* We first consider the complement universal co-Büchi automaton.
 * According to Filliot Jin Raskin CAV'09 this is equivalent to a K-universal co-Büchi condition
 * with K = 2 n (n^{2n+2} + 1).
 * We then determinize this K-universal co-Büchi into a safety game.
 *)


let to_safety aiger target = 
  let l = aig.Aiger.num_latches in 
  let size_counter = l + 4 + (1 lsl (l+1)) in  (* K < 2^{size+4+2^{size+1}} *)
  let det = Determinization.determinize aiger [??] in
  let coBuchi = coBuchi.to_safety det size_counter target in
  coBuchi
