type player = int

module PlayerMap : Map.S with type key = player

type payoff = string PlayerMap.t

type positive_formula = 
| LimInfLE of string * int
| LimSupGE of string * int
| Or of positive_formula * positive_formula
| PNeg of negative_formula
and negative_formula =
| LimInfGE of string * int
| LimSupLE of string * int
| And of negative_formula * negative_formula
| NNeg of positive_formula

type positive_negative_formula = 
| PF of positive_formula
| NF of negative_formula
    

type formula = (Coalition.t * positive_formula list * negative_formula list) list

val parse_positive_negative_formula : Genlex.token Stream.t -> positive_negative_formula

val lexer : char Stream.t -> Genlex.token Stream.t

val formula_of_string : string -> formula

val formula_to_string : formula -> string
val positive_formula_to_atomic_formula : positive_formula -> StateFormula.t

val negative_formula_to_atomic_formula : negative_formula -> StateFormula.t

(** returns a new game, controllables, uncontrollables, list of Buchi conditions and a coBuchi condition. *)
val formula_to_Buchi_coBuchi : Aiger.t -> Coalition.t -> formula -> (Aiger.t * Aiger.lit list * Aiger.lit list * Cudd.bdd list * Cudd.bdd)

(** The first BDD contains state with priority 2 and the second one, states with priority 3. Other states have priority 1. *)
val formula_to_parity : Aiger.t -> Coalition.t -> formula -> (Aiger.t * Aiger.lit list * Aiger.lit list * Cudd.bdd * Cudd.bdd)

val equilibria_region : Aiger.t -> Coalition.t -> formula -> Aiger.t * Attractor.t

(** Takes an aiger file and a formula *)
val test : string -> string -> unit 
