(* The aiger file should contain for each player i an output payoffi *)
(* The possible payoffs should be given in a separate file *)

module PlayerMap = LimitEquilibria.PlayerMap

let payoff_le c x = LimitEquilibria.LimInfLE (Printf.sprintf "payoff%d" c,x)
let payoff_ge c x = LimitEquilibria.LimInfGE (Printf.sprintf "payoff%d" c,x)

let try_payoff game payoff_vector =
  let payoff_equal = 
    PlayerMap.fold 
      (fun pl pay form -> 
	(Coalition.empty,[payoff_le pl pay],[payoff_ge pl pay])
	:: form
      ) payoff_vector []
  in
  
  let no_deviation pl pay = Coalition.singleton pl, [payoff_le pl pay] ,[] in

  let formula = List.rev_append payoff_equal (List.map (fun (x,pay) -> no_deviation x pay) (PlayerMap.bindings payoff_vector)) in

  let players = 
    PlayerMap.fold
      (fun pl _ co -> 
	Coalition.add pl co
      )
      payoff_vector Coalition.empty in

  print_endline (LimitEquilibria.formula_to_string formula);

  let aiger, eq = LimitEquilibria.equilibria_region game players formula in

  let initial = AigerBdd.initial_state aiger in
  let initial_winning = Cudd.bddRestrict (Attractor.latch_configuration eq) initial in
    
  (*print_endline (payoff_vector_to_string payoff_vector);*)
  if Cudd.value initial_winning = 1 
  then print_endline "realizable"
  else if Cudd.value initial_winning = 0
  then print_endline "unrealizable"
  else (Cudd.dumpDot ("initial_winning.out") initial_winning)


let test aiger_file payoff_file =
  let payoffs = 
    let file = open_in payoff_file in
    let rec aux accu =
      let int =
	try 
	  Some (int_of_string (input_line file))
	with _ -> None
      in match int with None -> accu | Some x -> aux (x :: accu)
    in 
    let res = aux [] in
    close_in file; 
    res
  in
  let aiger = Aiger.read_from_file aiger_file in
  let players = Coalition.of_aiger aiger in
  let rec aux payoff non_affected =
    if Coalition.is_empty non_affected 
    then try_payoff aiger payoff 
    else
      List.iter 
	(fun y ->
	  let p = Coalition.choose non_affected in
	  let new_payoff = PlayerMap.add p y payoff in
	  aux new_payoff (Coalition.remove p non_affected)
	) payoffs
  in 
  aux PlayerMap.empty players

let main = 
  if Filename.basename Sys.argv.(0) = "limitNash" ||
    Filename.basename Sys.argv.(0) = "limitNash.byte"
  then
    if Array.length Sys.argv < 3
    then
      Printf.printf "usage : %s aiger_file payoff_file\n" Sys.argv.(0)
    else
      test Sys.argv.(1) Sys.argv.(2)

