module Down = Antichain.DownwardClosed(struct type t = int let compare = compare end)
module Up = Antichain.UpwardClosed(struct type t = int let compare = compare end)

open Translation

let pareto_search game win =

  let set_objectives payoff =
    let tab = Hashtbl.create (List.length game.players) in
    List.iter 
      (fun p -> 
	if payoff.(p) = 0
	then Hashtbl.add tab p (game.objective p) 
	else Hashtbl.add tab p Aiger.aiger_false
      ) game.players;
    let objective p = try Hashtbl.find tab p with Not_found -> 
      Printf.printf "%d not found\n" p;
      raise Not_found
    in
    objective
  in 

  let maximum_player = List.fold_left max 0 game.players in
  let payoff = Array.make (maximum_player+1) 1 in

  let new_objective = set_objectives payoff in
  if win new_objective
  then Down.singleton payoff, Up.empty
  else

    let rec aux sat unsat = 
      List.fold_left
	(fun (sat,unsat) p -> 
	  if payoff.(p) = 0 then (sat,unsat)
	  else (
	    payoff.(p) <- 0;
	    let new_objective = set_objectives payoff in
	    let res =
	      if win new_objective
	      then aux (Down.insert sat payoff) unsat
	      else aux sat (Up.insert unsat payoff)
	    in
	    payoff.(p) <- 1;
	    res
	  )
	    
	) (sat,unsat) game.players
	
    in 
    aux Down.empty Up.empty
  
let call_solver game = 
  if game.objective 3 != Aiger.aiger_false
  then true else false


let main =
  if Array.length Sys.argv < 2 then failwith "syntax : main [filename]";

  let mgame = Translation.read_from_file Sys.argv.(1) in
  let game = Translation.deviator_game mgame in
  Aiger.write game stdout;
  
  let search_up, search_down = pareto_search mgame (fun obj -> call_solver {mgame with objective = obj}) in
  print_endline "upward sat:";
  Down.output string_of_int stdout search_up;
  print_newline ();
  print_endline "downward unsat:";
  Up.output string_of_int stdout search_down;
  print_newline ();
