let aiger_path = "../aiger-1.9.4/aiger-1.9.4/"

type t = AIG of Aiger.t

let import file = 
  let aig = 
    if Filename.check_suffix file ".smv" 
    then 
      let new_file = file^".aag" in
      let res = Sys.command (aiger_path^"smvtoaig <"^file^" >"^new_file) in
      if res = 0 then Printf.printf "wrote file %s\n" new_file;
      Aiger.read_from_file new_file 

    else if Filename.check_suffix file ".blif" 
    then 
      let new_file = file^".aag" in
      let res = Sys.command (aiger_path^"bliftoaig <"^file^" >"^new_file) in
      if res = 0 then Printf.printf "wrote file %s\n" new_file;
      Aiger.read_from_file new_file 

    else if Filename.check_suffix file ".aag" 
    then Aiger.read_from_file file 
    else failwith ("unknown extension : "^file)
  in AIG aig

let to_aiger m = match m with AIG a -> a

let write m file = 
  let a = match m with AIG a -> a in
  
  if Filename.check_suffix file ".smv" 
  then 
    let new_file = file^".aag" in
    let f = open_out new_file in Aiger.write a f; close_out f;
    Printf.printf "wrote file %s\n" new_file;
    print_newline ();
    let res = Sys.command (aiger_path^"aigtosmv <"^new_file^" >"^file) in ()
								       
  else if Filename.check_suffix file ".blif" 
  then 
    let new_file = file^".aag" in
    let f = open_out new_file in Aiger.write a f; close_out f;
    Printf.printf "wrote file %s\n" new_file;
    let res = Sys.command (aiger_path^"aigtoblif <"^new_file^" >"^file) in ()
									
  else if Filename.check_suffix file ".aag" 
  then     
    let f = open_out file in Aiger.write a f; close_out f
  else failwith ("unknown extension : "^file)
    
let variables = function
  | AIG a -> 
    let find = List.map (fun l -> Aiger.lit2symbol a l) in
    let interface = find a.Aiger.outputs in 
    let externals = find a.Aiger.inputs in 
    let privates = find (List.map fst a.Aiger.latches) in 
    (List.map Aiger.Symbol.to_string externals,List.map Aiger.Symbol.to_string privates,List.map Aiger.Symbol.to_string interface)

let external_var m = (function (e,_,_) -> e) (variables m)
let private_var m = (function (_,p,_) -> p) (variables m)
let interface_var m = (function (_,_,i) -> i) (variables m)

let synthesize m = failwith "synthesize not implemented"

let infos m =
  let (e,p,i) = variables m in
  print_endline "=== external variables: ===";
  List.iter print_endline e;
  print_endline "=== private variables: ===";
  List.iter print_endline p;
  print_endline "=== interface variables: ===";
  List.iter print_endline i

let compose m1 m2 =
  prerr_endline "warning: we should check compatibility";
  match m1,m2 with AIG a1, AIG a2 -> AIG (Aiger.compose a1 a2)

let hide vars m = 
  match m with 
  | AIG a -> AIG(List.fold_left Aiger.hide a (List.map Aiger.Symbol.of_string vars))


type parameter = Int of int | String of string | Bool of bool

let parametric parameters f = 
  let table = Hashtbl.create (List.length parameters) in
  let add p v = Hashtbl.replace table p v in
  let find p = Hashtbl.find table p in
  let specs = 
    List.map (fun (name,p) -> 
      add name p; 
      match p with
      | Int i -> 
	name,Arg.Int (fun j -> add name (Int j)), "integer parameter "^name
      | String s -> 
	name,Arg.String (fun j -> add name (String j)), "string parameter "^name
      | Bool b -> 
	name,Arg.Bool (fun j -> add name (Bool j)), "boolean parameter "^name
    ) parameters
  in
  Arg.parse specs ignore "This is a parametric module";
  let m = f find in
  let a = to_aiger m in 
  Aiger.write a stdout


let main = 
 let lexer = Genlex.make_lexer ["help";"import";"="; "infos";"("; ")"; "quit";"write"]  in
 let modules = Hashtbl.create 10 in
 let find m = 
   try Hashtbl.find modules m
   with Not_found ->
     Printf.eprintf "module %s not found\n" m;
     raise Not_found
 in

 print_endline "type a command or 'help' to for a list of command";

  let rec loop () =
    let line = read_line () in
    let stream = lexer (Stream.of_string line) in
    let parse_expr = parser
      | [< 'Genlex.Kwd "import"; 'Genlex.String file >] -> import file
      | [<'Genlex.Kwd "compose"; 'Genlex.Ident name1; 'Genlex.Ident name2 >] -> compose (find name1) (find name2)
    in
    
    let parse = parser 
      | [< 'Genlex.Kwd "help" >] -> 
	print_endline "help\n\tDisplay this message.\n";
	print_endline "quit\n\tExit the program.\n";
	print_endline "m = import \"filename\"\n\tRead the module in file [filename] and store it under the name [m].\n";
	print_endline "infos m\n\tDisplay informations about module [m].\n";
	print_endline "m = compose m1 m2\n\tCombine modules [m1] and [m2] into a new module [m2].\n";
	print_endline "write m \"filename\"\n\tWrite module [m] file [filename].\n";
	loop ()
	
      | [< 'Genlex.Kwd "quit" >] -> ()
      | [< 'Genlex.Ident name; 'Genlex.Kwd "="; expr = parse_expr>]
	-> Hashtbl.add modules name expr; loop ()
	  
      | [< 'Genlex.Kwd "infos"; 'Genlex.Ident name >] ->
	infos (find name); loop ()

      | [< 'Genlex.Kwd "write"; 'Genlex.Ident name; 'Genlex.String file >] -> 
	write (find name) file; loop ()

      | [< >] -> loop ()
    in 
    parse stream
    (*try (parse stream)
     with _ -> Printf.printf "parsing error\n"*);
  in

  loop ()
  

