type t

(** = High level manipulation of modules = *)
(** Read a module from an AIGER file. *)
val import : string -> t

(** Write a module to an AIGER, SMV or BLIF file, according to the extension of the filename. *)
val write : t -> string -> unit

(** Display informations about the module. *)
val infos : t -> unit

(** Compose two modules. *)
val compose : t -> t -> t

(** Hide interface variable to make them private. *)
val hide : string list -> t -> t

(** Rename variables according to the given list. *)
(* val rename : (string * string) list -> t -> t *)

(** Synthesize a module from its specification. *)
val synthesize : t -> string -> t

(** = More detailed operations = *)
val external_var : t -> string list
val private_var : t -> string list
val interface_var : t -> string list

val variables : t -> (string list * string list * string list)



