(** Parity games.
    The highest priority seen infinitely often determines the winner.
    If it is even, the controller wins.
    If it is odd, the controller loses. *)

(** Solve a game with 2 priorities.
    The 4-th argument is the set of states that have priority 2.
    If the 5-th argument is true then the controller is weak (ie plays first).
    The last argument is the region to which we want to restrict the computation.
    To not restrict it use [Attractor.tt()]
*)
val parity2 : AigerBdd.t -> Aiger.lit list -> Aiger.lit list -> Cudd.bdd -> bool -> Attractor.t -> Attractor.t


(** [parity3 aiger controllables uncontrollables parity2 parity3 weak restriction]
    Solves a game with 3 priorities. 
    The [parity2] argument is the set of states that have priority 2.
    The [parity3] argument is the set of states that have priority 3.
    Other states are assumed to have priority 1.
    The last argument is a restriction on the states. 
*)
val parity3 : AigerBdd.t ->
           Aiger.lit list ->
           Aiger.lit list -> Cudd.bdd -> Cudd.bdd -> bool -> Attractor.t -> Attractor.t

(** In the AIGER file, the states with priority 2 should be given by an output called PARITY2 or FAIRNESS and states with priority 3 with an output PARITY3 *)
val test : Aiger.t -> unit

