(* Converts psl specification to safety games.
   A bound can be given to specify how many visits we allow in the co-Buchi target. *)

(* let _ = Common.display_debug := true*)

let psl2ba formula = 
  let ext_less = "tmp_psl2ba" in (* Filename.chop_extension file in*)
  let smv_file = ext_less^".smv" in
  if !Common.display_debug then Printf.fprintf stderr "../psl2ba_2010-01-26/psl2ba/bin/psl2ba -smv %S >%s\n" formula smv_file;
  Sys.command
    (Printf.sprintf "../psl2ba_2010-01-26/psl2ba/bin/psl2ba -smv %S >%s" formula smv_file);
  smv_file



let smv2speculog smv_name = 
  let smv_channel = open_in smv_name in
  let _,smv = Smv.read smv_channel in
  close_in smv_channel;
  let ext_less = Filename.chop_extension smv_name in
  let spe_file = ext_less^".spe" in
  let outch = open_out spe_file in
  Smv.to_speculog outch None smv;
  close_out outch;
  spe_file

let to_safety aiger bound =
  (* add counter *)
  let kcobuchi = CoBuchi.to_safety_non_det aiger bound (Aiger.lit2symbol aiger (List.hd aiger.Aiger.outputs)) in  
  Common.debug "K co-Buchi game generated";
(* determinize *)
  Cudd.init kcobuchi.Aiger.maxvar;
  let det = Determinization.determinize kcobuchi in
  Common.debug "K co-Buchi game determinized";
  det

let test formula =
  let smv_name = psl2ba formula in
  Common.debug "smv file written";
  let spe_name = smv2speculog smv_name in
  let aig = Speculog.to_aig spe_name in
  if !Common.display_debug then Aiger.write aig stdout;
  Common.debug "aiger file written";
  let rec loop bound = 
    Aiger.write (to_safety aig bound) stdout
  in loop 2

let main = 
  if Filename.basename Sys.argv.(0) = "psl" 
  then   
    if Array.length Sys.argv < 2
    then Printf.printf "usage : %s psl_formula\n" Sys.argv.(0)
    else test Sys.argv.(1)
  
