let print_state aiger state = 
  let valuation = AigerBdd.extract_valuation aiger state
    (List.rev_append (Aiger.outputs aiger) (Aiger.latches aiger))
  in
  List.iter
    (fun (name,i) -> 	  
      Printf.printf "%s : %d\n" name i;
    ) valuation

let input_inputs aiger =
  List.fold_left
    (fun state name -> 
      let size = Aiger.size_symbol aiger name in
      Printf.printf "%s[%d:0]?\n> " name (size-1);
      let b = read_int () in 
      if size = 1 then 
	Aiger.LitMap.add (Aiger.symbol2lit aiger (name,None)) (b=1) state
      else
	let rec loop state i = 
	  if i = size then state 
	  else
	    let new_state = 
	      if (b lsr i) mod 2 = 1 
	      then 
		(
		  if !Common.display_infos 
		  then Printf.printf "%s<%d> = true\n" name i;
		  Aiger.LitMap.add (Aiger.symbol2lit aiger (name,Some i)) true state
		)
	      else 
		(
		  if !Common.display_infos
		  then Printf.printf "%s<%d> = false\n" name i;
		  Aiger.LitMap.add (Aiger.symbol2lit aiger (name,Some i)) false state
		)
	    in loop new_state (i+1)

	in loop state 0
    ) 
    Aiger.LitMap.empty
    (Aiger.inputs aiger)


let input_from_channel inputs inch =
  let i = ref (-1) in
  let line = 
    let rec loop () = 
      let x = input_line inch in
      if x.[0] <> '0' && x.[0] <> '1' 
      then (print_endline x; loop ()) else x 
    in loop ()
  in
  
  List.fold_left
    (fun valuation lit -> 
      incr i;
      Aiger.LitMap.add lit (if line.[!i] = '0' then false else true) valuation
    )
    Aiger.LitMap.empty
    inputs

let main =
  if Array.length Sys.argv < 2 then prerr_endline "usage: simulation <aiger file>";
  Printf.printf "reading from %s\n" Sys.argv.(1);
  let aiger = Aiger.read_from_file Sys.argv.(1) in
  AigerBdd.init aiger; 
  let initial_state = AigerBdd.initial_state aiger in
  let states = AigerBdd.bdd_to_valuations initial_state (aiger.Aiger.outputs @ List.map fst aiger.Aiger.latches) in
  let state = List.hd states in
  let game = AigerBdd.of_aiger aiger in
  let input_file = 
    if Array.length Sys.argv < 3 
    then None else Some (open_in Sys.argv.(2))
  in
  Common.infos "updates computed";
  print_state aiger state;
  let rec loop state = 
    try 
      let inputs = match input_file with 
	| None -> input_inputs aiger 
	| Some inch -> input_from_channel aiger.inputs inch
      in
      let new_state = List.hd (AigerBdd.successor_states game state inputs) in
      print_state aiger new_state;
      loop new_state
    with
    | End_of_file -> state
    | Failure x -> Printf.printf "exiting (%s)\n" x; state
  in
  
  loop state;
  match input_file with Some x -> close_in x | None -> ()
