module StringSet : Set.S with type elt = string

type t = 
| AF_true 
| AF_false
| AF_or of t * t
| AF_and of t * t
| AF_not of t
| AF_equal of string * int

val of_bool : bool -> t
val of_equal : string -> int -> t
val of_greater_equal : string -> int -> t
val of_smaller_equal : string -> int -> t
val conjunction : t -> t -> t
val disjunction : t -> t -> t
val implication : t -> t -> t
val negation : t -> t


val to_string : t -> string 
val to_short_string : t -> string 
val to_name : t -> string
val of_string : string -> t

val compare : t -> t -> int

  (** Generates a BDD representing the set of states in which 
      the formula is evaluated to true. The first argument
      associates to a variable name, an array containing the
      litterals encoding it in the aiger file. *)
val to_bdd : (string -> Aiger.lit array) -> t -> Cudd.bdd

val parse : Genlex.token Stream.t -> t

val tokens : string list 

val lexer : char Stream.t -> Genlex.token Stream.t

  (** Set of names that appear in the formula. *)
val names : t -> StringSet.t


