module StringSet : Set.S with type elt = string

module AFSet : Set.S with type elt = StateFormula.t


module Formula :
sig
  type temporal_formula = 
  | TF_eventually of StateFormula.t
  | TF_always of StateFormula.t
      
  type t = 
  | F_or of t * t
  | F_and of t * t
  | F_implies of t * t
  | F_not of t
  | F_temp of temporal_formula
      
  val to_string : t -> string
  val to_short_string : t -> string
  val to_name : t -> string
  val parse : string -> t
  val compare : t -> t -> int
end

module NormalizedFormula :
sig
  type t
  val of_formula : Formula.t -> t
  val compare : t -> t -> int
  val to_string : t -> string 
  (** Set of atomic formulas that appear in the given formula. *)
  val atomic_formulas : t -> AFSet.t
end

(** Construct an automaton from a formula. The automaton 
    keeps track of the atomic formula that have been 
    evaluated to true since the begining.
    The first argument gives the size (in number of bit) of the variable present in the formula. *)
val formula_to_automaton : (string -> int) -> Formula.t -> Aiger.t


(** [winning_region aiger controllables uncontrollables formula weak]
    returns the product game with the automaton and the winning region *)
val winning_region : Aiger.t -> Aiger.lit list -> Aiger.lit list -> Formula.t -> bool -> (Aiger.t * Attractor.t)


(** strong set to true means the controller should win even without seeing inputs of the environment *)
val test : Aiger.t -> ?strong:bool -> string -> unit
