module Form = WeakMullerEquilibria
module PlayerMap = Form.PlayerMap

let try_payoff game payoff_definition payoff_vector = 
  let payoff_equal = 
    let c,pay_c = PlayerMap.choose payoff_vector in
    PlayerMap.fold 
      (fun pl pay form -> Form.PF_and (Form.PF_compare (pl,Form.Equal,pay), form))
      (PlayerMap.remove c payoff_vector) (Form.PF_compare (c,Form.Equal,pay_c))
  in

  let no_deviation pl pay = Form.Coalition.singleton pl, Form.PF_compare (pl,Form.SmallerEq,pay) in
  let formula = (Form.Coalition.empty, payoff_equal) :: List.map (fun (x,pay) -> no_deviation x pay) (PlayerMap.bindings payoff_vector) in

  Common.debug (Form.formula_to_string formula);

  let g,safe = Form.equilibria_region game payoff_definition formula in

  let initial = AigerBdd.initial_state g in
  let initial_winning = Cudd.bddRestrict safe initial in
    
  print_endline (Form.payoff_vector_to_string payoff_vector);
  if Cudd.value initial_winning = 1 
  then print_endline "realizable"
  else if Cudd.value initial_winning = 0
  then print_endline "unrealizable"
  else (print_endline "problem determing whether the initial state is winning";
	Cudd.dumpDot ("initial_winning.out") initial_winning)


let main = 
  if Array.length Sys.argv < 3
  then Printf.printf "usage : %s aiger_file payoff_file\n" Sys.argv.(0);
  Cudd.init 100;
  let game = Translation.read_from_file Sys.argv.(1) in
  let payoff_definition = Form.read_payoff_file Sys.argv.(2) in


  List.iter (try_payoff game payoff_definition) (Form.possible_payoffs payoff_definition);
  Common.debug "finished"
    
