let compare_to_weak_Muller payoff_vector payoff_definition (player,comparison,int) =
  let payoff_list =
    List.fold_left (fun a (fj,j) -> j :: a) [] (try PlayerMap.find player payoff_definition with _ -> failwith "payoff_definition")
  in

  (* return a formula or raise an exception if the formula is trivial *)
  let aux k = function
    | E_int i -> 
      if i >= k then raise (TrivialFormula true)
      else raise (TrivialFormula false)
	
    | E_payoff (p,i) -> 
      if PlayerMap.find p payoff_vector + i >= k
      then raise (TrivialFormula true)
      else raise (TrivialFormula false)
	
    | E_payoff_prime (p,i) ->     *)

  let rec aux_prime accu = function 
    | [] -> accu
    | (fj,j) :: s -> 
      if (comparison_function comparison) j int
      then 
	(
	  match accu with 
	  | Some f -> aux_prime (Some (WeakMuller.F_or (f,fj))) s
	  | None -> aux_prime (Some fj) s)
      else aux_prime accu s
  in 
  match aux_prime None (PlayerMap.find player payoff_definition) with
  | None -> raise (TrivialFormula false)
  | Some x -> x

in 

  let aux_conj k =  
    try 
      let a1 = aux (k+1) expr1 in
      try 
	let a2 = aux k expr2 in
	WeakMuller.F_and (a2,WeakMuller.F_not a1) 
      with 
      | TrivialFormula true -> WeakMuller.F_not a1
      | TrivialFormula false -> raise (TrivialFormula false)
    with 
    | (TrivialFormula true) -> raise (TrivialFormula false)
    | (TrivialFormula false) -> aux k expr2
  in
  let payoffs = payoff_list expr1 in
  let res_opt = 
    List.fold_left
      (fun f_opt k -> 
	try let a = aux_conj k in match f_opt with 
	| None -> Some a
	| Some f -> Some (WeakMuller.F_or (f,a))
	with TrivialFormula false -> f_opt)
      None payoffs
  in match res_opt with 
  | Some x -> x
  | None -> raise (TrivialFormula false)




let formula_to_weak_Muller payoff_vector payoff_definition formula = 
  let players = PlayerMap.fold (fun p i c -> Coalition.add p c) payoff_vector Coalition.empty in
  let not_deviator x = WeakMuller.F_temp (WeakMuller.TF_always (WeakMuller.AF_equal ((Translation.deviator x),0))) in

  let rec aux = function 
    | F_or (f1,f2) -> 
      (try let r1 = aux f1 in
	   try let r2 = aux f2 in WeakMuller.F_or(r1,r2)
	   with TrivialFormula false -> r1
       with TrivialFormula false -> aux f2)

    | F_and (f1,f2) -> 
      (try let r1 = aux f1 in
	   try let r2 = aux f2 in WeakMuller.F_and(r1,r2)
	   with TrivialFormula true -> r1
       with TrivialFormula true -> aux f2)

    | F_forall_strategies (c,pf) -> 
      let elements = Coalition.elements (Coalition.diff players c) in
      match elements with 
    | [] -> raise (TrivialFormula true)
    | hd :: tl ->
      let inclusion = (* always deviator hd = 0 /\ always deviator tl = 0 *)
	List.fold_left (fun x p -> WeakMuller.F_and (x,not_deviator p)) (not_deviator hd) tl
      in
      WeakMuller.F_implies (inclusion,payoff_formula_to_weak_Muller payoff_vector payoff_definition pf)
  in 

  let Some(no_deviation) = 
    Coalition.fold 
      (fun p -> 
	function 
	| None -> Some (not_deviator p)
	| Some x ->  Some(WeakMuller.F_and (x,not_deviator p))) 
      players None 
  in
  
  let Some (equal_payoff) = 
    let aux p = PF_and
      (PF_smaller_eq (E_payoff (p,0), E_payoff_prime (p,0)),
       PF_smaller_eq (E_payoff_prime (p,0), E_payoff (p,0)))
    in
    Coalition.fold (fun p -> 
      function 
      | None -> Some (aux p)
      | Some x ->  Some(PF_and (x,aux p) )
    ) players None
  in

  let no_deviation_formula = WeakMuller.F_implies(no_deviation,payoff_formula_to_weak_Muller payoff_vector payoff_definition equal_payoff) in

  try
    let main_formula = aux formula in
    WeakMuller.F_and (main_formula,no_deviation_formula)
  with TrivialFormula true -> no_deviation_formula



let main = 
  let rec aux_payoff payoff_vector = 
    try 
      print_endline "------------------";
      print_endline (payoff_vector_to_string payoff_vector);

  in


  List.iter aux_payoff (possible_payoffs payoff_definition)
