
let test () = 
  if Array.length Sys.argv < 2 
  then Printf.printf "usage : %s <file>\n" Sys.argv.(0);
  let aig = Aiger.read_from_file Sys.argv.(1) in
  Cudd.init (aig.Aiger.num_inputs+2*aig.Aiger.num_latches+2);



  let make_controllable a inputs = 
    let aux_rename lit = 
      let sym = Aiger.lit2symbol a lit in
      let sym_string = Aiger.Symbol.to_string sym in
      if List.mem lit inputs then sym,(sym_string,None)
      else sym , ("uncontrollable_"^sym_string,None)
    in
    let renaming = List.map aux_rename a.Aiger.inputs in
    Aiger.rename a renaming
  in

  let rec loop accu = function 
    | [] -> ()
    | a :: s ->
       let time = Sys.time () in
       let new_controllables = a :: accu in 
       print_string "Making : ";
       List.iter (fun x -> Printf.printf "%s " (Aiger.Symbol.to_string (Aiger.lit2symbol aig x))) new_controllables;
       print_endline " controllable";
       let aig_c = make_controllable aig new_controllables in
       let strat,r1,r2 = SimulationRelation.simulation aig_c in
       Printf.printf "computation time: %f\n" (Sys.time () -. time);
       Cudd.dumpDot ("simulation_"^string_of_int (List.length s)^".dot") strat;
       loop new_controllables s

  in loop [] (List.filter (fun x -> Attractor.controllable_name (fst (Aiger.lit2symbol aig x))) aig.Aiger.inputs)





let main =
  if Filename.check_suffix Sys.argv.(0) "wordDominance" 
  then test ()  
